Added the timing of the graph building and tracking in seconds.
[mtp.git] / mtp.cc
diff --git a/mtp.cc b/mtp.cc
index ce9e56b..36f09f1 100644 (file)
--- a/mtp.cc
+++ b/mtp.cc
 
-///////////////////////////////////////////////////////////////////////////
-// This program is free software: you can redistribute it and/or modify  //
-// it under the terms of the version 3 of the GNU General Public License //
-// as published by the Free Software Foundation.                         //
-//                                                                       //
-// This program is distributed in the hope that it will be useful, but   //
-// WITHOUT ANY WARRANTY; without even the implied warranty of            //
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      //
-// General Public License for more details.                              //
-//                                                                       //
-// You should have received a copy of the GNU General Public License     //
-// along with this program. If not, see <http://www.gnu.org/licenses/>.  //
-//                                                                       //
-// Written by and Copyright (C) Francois Fleuret                         //
-// Contact <francois.fleuret@idiap.ch> for comments & bug reports        //
-///////////////////////////////////////////////////////////////////////////
-
-// Multi-Tracked Path
-
-// Takes the graph description file as input and produces a dot file.
-
-// EXAMPLE: ./mtp ./graph2.txt  | dot -T pdf -o- | xpdf -
+/*
+ *  mtp is the ``Multi Tracked Paths'', an implementation of the
+ *  k-shortest paths algorithm for multi-target tracking.
+ *
+ *  Copyright (c) 2012 Idiap Research Institute, http://www.idiap.ch/
+ *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
+ *
+ *  This file is part of mtp.
+ *
+ *  mtp is free software: you can redistribute it and/or modify it
+ *  under the terms of the GNU General Public License version 3 as
+ *  published by the Free Software Foundation.
+ *
+ *  mtp is distributed in the hope that it will be useful, but WITHOUT
+ *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
+ *  License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with selector.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ */
 
 #include <iostream>
 #include <fstream>
-#include <cmath>
-#include <stdio.h>
-#include <stdlib.h>
-#include <float.h>
 
 using namespace std;
 
-#include "mtp_graph.h"
-#include "tracker.h"
-
-//////////////////////////////////////////////////////////////////////
-
-void find_best_paths(int nb_vertices,
-                     int nb_edges, int *ea, int *eb, scalar_t *el,
-                     int source, int sink,
-                     int *result_edge_occupation) {
-  MTPGraph graph(nb_vertices, nb_edges, ea, eb, source, sink);
-  graph.find_best_paths(el, result_edge_occupation);
-}
-
-//////////////////////////////////////////////////////////////////////
+#include "mtp_tracker.h"
 
 int main(int argc, char **argv) {
-  // int nb_locations = 6;
-  // int nb_time_steps = 5;
-
-  // {
-    // Tracker tracker(nb_time_steps, nb_locations);
-
-    // for(int l = 0; l < nb_locations; l++) {
-      // for(int k = 0; k < nb_locations; k++) {
-        // tracker.set_allowed_motion(l, k, abs(l - k) <= 1);
-      // }
-    // }
-
-    // for(int t = 0; t < nb_time_steps; t++) {
-      // for(int l = 0; l < nb_locations; l++) {
-        // tracker.set_detection_score(t, l,
-                                    // (drand48() < 0.9 ? -1.0 : 1.0) + drand48() * 0.1 - 0.05);
-      // }
-      // tracker.set_detection_score(t, 0,
-                                  // (drand48() < 0.9 ? 1.0 : -1.0) + drand48() * 0.1 - 0.05);
-    // }
-
-    // tracker.build_graph();
-    // tracker.track();
-  // }
-
-  // exit(0);
+  time_t start_time, end_time;
 
   if(argc < 2) {
-    cerr << argv[0] << " <graph file>" << endl;
+    cerr << argv[0] << " <tracker file>" << endl;
     exit(EXIT_FAILURE);
   }
 
-  ifstream *file = new ifstream(argv[1]);
+  ifstream *in_tracker = new ifstream(argv[1]);
 
-  int nb_edges, nb_vertices;
-  int source, sink;
+  if(in_tracker->good()) {
 
-  if(file->good()) {
+    MTPTracker *tracker = new MTPTracker();
 
-    (*file) >> nb_vertices >> nb_edges;
-    (*file) >> source >> sink;
+    cout << "Reading " << argv[1] << "." << endl;
+    tracker->read(in_tracker);
 
-    scalar_t *edge_lengths = new scalar_t[nb_edges];
-    int *vertex_from = new int[nb_edges];
-    int *vertex_to = new int[nb_edges];
-    int *result_edge_occupation = new int[nb_edges];
+    cout << "Building the graph ... "; cout.flush();
+    start_time = time(0);
+    tracker->build_graph();
+    end_time = time(0);
+    cout << "done (" << end_time - start_time << "s)." << endl;
 
-    for(int e = 0; e < nb_edges; e++) {
-      (*file) >> vertex_from[e] >> vertex_to[e] >> edge_lengths[e];
-    }
+    cout << "Tracking ... "; cout.flush();
+    start_time = time(0);
+    tracker->track();
+    end_time = time(0);
+    cout << "done (" << end_time - start_time << "s)." << endl;
 
-    find_best_paths(nb_vertices, nb_edges,
-                    vertex_from, vertex_to, edge_lengths,
-                    source, sink,
-                    result_edge_occupation);
+    ofstream out_traj("result.trj");
+    tracker->write_trajectories(&out_traj);
+    cout << "Wrote result.trj" << endl;
 
-    // dot_print(nb_vertices, nb_edges,
-              // vertex_from, vertex_to, edge_lengths,
-              // source, sink,
-              // result_edge_occupation);
+    ofstream out_dot("graph.dot");
+    tracker->print_graph_dot(&out_dot);
+    cout << "Wrote graph.dot" << endl;
 
-    delete[] result_edge_occupation;
-    delete[] edge_lengths;
-    delete[] vertex_from;
-    delete[] vertex_to;
+    delete tracker;
 
   } else {
 
     cerr << "Can not open " << argv[1] << endl;
-
-    delete file;
     exit(EXIT_FAILURE);
 
   }
 
-  delete file;
+  delete in_tracker;
+
   exit(EXIT_SUCCESS);
 }